##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

'''Patches App.Product so that a manage_refresh tab is available for
all products that are loaded from the filesystem.
$Id: RefreshPatch.py,v 1.6 2001/02/12 20:26:45 shane Exp $
'''

import App.Product
import Globals
from Globals import HTMLFile, PersistentMapping
import Products
from os import path
import sys
from ZODB.Connection import Connection
from ZODB.ZApplication import ZApplicationWrapper, Cleanup
import RefreshFuncs
from string import join


# Don't let this module be refreshed because it is a patch.
__refresh_module__ = 0

origInitializeProduct = App.Product.initializeProduct

orig_setDB = Connection._setDB


def performPatch():

    ProductModule = App.Product
    ProductClass = App.Product.Product

    def initializeProduct(productp, name, home, app):
        # Replacement for App.Product.initializeProduct().
        # Adds the Refresh tab to the product management screens.
        product = origInitializeProduct(productp, name, home, app)
        if product.__dict__.has_key('manage_options'):
            found = 0

            # Remove refresh tabs.
            # Uncomment and restart Zope if RefreshPatch causes
            # unexpected problems.
            #newOptions = []
            #for option in product.manage_options:
            #    if not option.has_key('label') or \
            #       option['label'] != 'Refresh':
            #        newOptions.append(option)
            #product.manage_options = tuple(newOptions)
            #found = 1

            # Ensure this product has a refresh tab.
            for option in product.manage_options:
                if option.has_key('label') and \
                   option['label'] == 'Refresh':
                    # Refresh tab already present.
                    found = 1
                    break
            if not found:
                product.manage_options = product.manage_options + (
                    {'label':'Refresh', 'action':'manage_refresh'},
                    )
        return product

    ProductModule.initializeProduct = initializeProduct

    ProductClass._refresh_dtml = HTMLFile('refresh', globals())

    def _readRefreshTxt(self):
        refresh_txt = None
        for productDir in Products.__path__:
            p = path.join(productDir, self.id, 'refresh.txt')
            if not path.exists(p):
                p = path.join(productDir, self.id, 'REFRESH.txt')
            if not path.exists(p):
                p = path.join(productDir, self.id, 'REFRESH.TXT')
            if path.exists(p):
                try:
                    file = open(p)
                    text = file.read()
                    file.close()
                    refresh_txt = text
                    break
                except:
                    # Not found here.
                    pass
        return refresh_txt

    ProductClass._readRefreshTxt = _readRefreshTxt

    def manage_refresh(self, REQUEST, reload_exception=None,
                       manage_tabs_message=None):
        '''
        Displays the refresh management screen.
        '''
        error_type = error_value = error_tb = None
        if reload_exception is not None:
            # Prepare the exception for display.
            try:
                exc = reload_exception
                if hasattr(exc[0], '__name__'):
                    error_type = exc[0].__name__
                else:
                    error_type = str(exc[0])
                error_value = str(exc[1])

                import traceback
                try:
                    from cStringIO import StringIO
                except:
                    from StringIO import StringIO

                f=StringIO()
                traceback.print_tb(exc[2], 200, f)
                error_tb = f.getvalue()
            finally:
                exc = reload_exception = None
            if not manage_tabs_message:
                manage_tabs_message = 'An exception occurred.'

        refresh_txt = self._readRefreshTxt()

        # Read the persistent refresh information.
        auto = RefreshFuncs.isAutoRefreshEnabled(self._p_jar, self.id)
        deps = RefreshFuncs.getDependentProducts(self._p_jar, self.id)

        # List all product modules.
        mods = RefreshFuncs.listRefreshableModules(self.id)
        loaded_modules = []
        prefix = 'Products.%s' % self.id
        prefixdot = prefix + '.'
        lpdot = len(prefixdot)
        for name, module in mods:
            if name == prefix or name[:lpdot] == prefixdot:
                name = name[lpdot:]
                if not name:
                    name = '__init__'
            loaded_modules.append(name)

        all_auto = RefreshFuncs.listAutoRefreshableProducts(self._p_jar)
        auto_other = filter(lambda productId, myId=self.id:
                            productId != myId, all_auto)

        # Return rendered DTML.
        return self._refresh_dtml(self, REQUEST,
                                  id=self.id,
                                  refresh_txt=refresh_txt,
                                  error_type=error_type,
                                  error_value=error_value,
                                  error_tb=error_tb,
                                  auto_refresh_enabled=auto,
                                  auto_refresh_other=auto_other,
                                  dependent_products=deps,
                                  loaded_modules=loaded_modules,
                                  manage_tabs_message=manage_tabs_message,
                                  management_view='Refresh')

    ProductClass.manage_refresh = manage_refresh

    def manage_performRefresh(self, REQUEST=None):
        '''
        Attempts to perform a refresh operation.  self is the Product
        object.
        '''
        if self._readRefreshTxt() is None:
            raise 'Unauthorized', 'refresh.txt not found'
        try:
            try:
                exc = None
                message = None
                RefreshFuncs.performFullRefresh(
                    self._p_jar, self.id, raise_exc=1)
                RefreshFuncs.updateGlobalClassesTimestamp()
                message = 'Product refreshed.'
            except:
                exc = sys.exc_info()
            if REQUEST is not None:
                return self.manage_refresh(REQUEST, reload_exception=exc,
                                           manage_tabs_message=message)
        finally:
            exc = None

    ProductClass.manage_performRefresh = manage_performRefresh

    def manage_enableAutoRefresh(self, enable=0, REQUEST=None):
        '''
        Changes the auto refresh flag for this product.
        '''
        if self._readRefreshTxt() is None:
            raise 'Unauthorized', 'refresh.txt not found'
        RefreshFuncs.enableAutoRefresh(self._p_jar, self.id, enable)
        if enable:
            message = 'Enabled auto refresh.'
        else:
            message = 'Disabled auto refresh.'
        if REQUEST is not None:
            return self.manage_refresh(REQUEST, manage_tabs_message=message)

    ProductClass.manage_enableAutoRefresh = manage_enableAutoRefresh

    def manage_selectDependentProducts(self, selections=(), REQUEST=None):
        '''
        Selects which products to refresh simultaneously.
        '''
        if self._readRefreshTxt() is None:
            raise 'Unauthorized', 'refresh.txt not found'
        RefreshFuncs.setDependentProducts(self._p_jar, self.id, selections)
        if REQUEST is not None:
            return self.manage_refresh(REQUEST)

    ProductClass.manage_selectDependentProducts = (
        manage_selectDependentProducts)

    # Reinitialize permissions for App.Product.Product.
    Globals.default__class_init__(ProductClass)
    
    def _setDB(self, odb):
        '''
        This hook provides the opportunity to clear the cache for
        each connection that needs to see the updated class definitions.
        '''
        rval = orig_setDB(self, odb)
        RefreshFuncs.mayClearCache(self)
        return rval

    Connection._setDB = _setDB

    def __bobo_traverse__(self, REQUEST=None, name=None):
        db, aname, version_support = self._stuff
        if version_support is not None and REQUEST is not None:
            version=REQUEST.get(version_support,'')
        else: version=''
        conn=db.open(version)

        self._autoRefresh(conn)

        # arrange for the connection to be closed when the request goes away
        cleanup=Cleanup()
        cleanup.__del__=conn.close
        REQUEST._hold(cleanup)

        conn.setDebugInfo(REQUEST.environ, REQUEST.other)
        
        v=conn.root()[aname]

        if name is not None:
            if hasattr(v, '__bobo_traverse__'):
                return v.__bobo_traverse__(REQUEST, name)
            
            if hasattr(v,name): return getattr(v,name)
            return v[name]
        
        return v

    ZApplicationWrapper.__bobo_traverse__ = __bobo_traverse__

    def _autoRefresh(self, conn):
        auto_refresh_ids = RefreshFuncs.checkAutoRefresh(conn)
        if auto_refresh_ids:
            RefreshFuncs.updateGlobalClassesTimestamp()
            RefreshFuncs.finishAutoRefresh(conn, auto_refresh_ids)
            get_transaction().commit()
            RefreshFuncs.mayClearCache(conn)
            get_transaction().begin()
            return 1
        return 0

    ZApplicationWrapper._autoRefresh = _autoRefresh


performPatch()

